"use strict";

var DefaultEventsStrategy = require("./events_strategy"),
    each = require("./utils/iterator").each,
    isPlainObject = require("./utils/type").isPlainObject;

/**
 * @name EventsMixin
 * @publicName EventsMixin
 * @module core/events_mixin
 * @export default
 * @hidden
 */
module.exports = {
    ctor: function ctor() {
        this._events = {};
        this.setEventsStrategy(new DefaultEventsStrategy(this));
    },

    setEventsStrategy: function setEventsStrategy(strategy) {
        if (typeof strategy === "function") {
            strategy = strategy(this);
        }

        this._eventsStrategy = strategy;
    },

    hasEvent: function hasEvent(eventName) {
        return this._eventsStrategy.hasEvent(eventName);
    },

    fireEvent: function fireEvent(eventName, eventArgs) {
        this._eventsStrategy.fireEvent(eventName, eventArgs);
        return this;
    },

    /**
     * @name EventsMixinMethods_on
     * @publicName on(eventName, eventHandler)
     * @param1 eventName:string
     * @param2 eventHandler:function
     * @return object
     */
    /**
     * @name EventsMixinMethods_on
     * @publicName on(events)
     * @param1 events:object
     * @return object
     */
    on: function on(eventName, eventHandler) {
        if (isPlainObject(eventName)) {
            each(eventName, function (e, h) {
                this.on(e, h);
            }.bind(this));
        } else {
            this._eventsStrategy.on(eventName, eventHandler);
        }

        return this;
    },

    /**
     * @name EventsMixinMethods_off
     * @publicName off(eventName)
     * @param1 eventName:string
     * @return object
     */
    /**
     * @name EventsMixinMethods_off
     * @publicName off(eventName, eventHandler)
     * @param1 eventName:string
     * @param2 eventHandler:function
     * @return object
     */
    off: function off(eventName, eventHandler) {
        this._eventsStrategy.off(eventName, eventHandler);
        return this;
    },

    _disposeEvents: function _disposeEvents() {
        this._eventsStrategy.dispose();
    }
};